﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.Globalization


Public Class AspectNode
	Inherits DiagramNode
	Public Sub New(ByVal causeNode As CauseNode)
		Me.causeNode = causeNode

		Stroke = Brushes.Blue
		StrokeThickness = 3
	End Sub

	Public Overrides Function GetRepaintRect(ByVal includeConnected As Boolean) As Rect
		Dim baseRect As Rect = MyBase.GetRepaintRect(includeConnected)
		baseRect.Inflate(2, 2)
		Return baseRect
	End Function

	Public Overrides Sub Draw(ByVal graphics As DrawingContext, ByVal options As MindFusion.Diagramming.Wpf.RenderOptions)
		Dim code As Integer = AscW("a"c) + Math.Max(0, (If(ZIndex = 0, Parent.Items.Count, ZIndex)) - 1)
		Dim c As Char = ChrW(code)
		Dim p1 As Point = Bounds.Location
		p1.X += Bounds.Width / 2

		Dim p2 As Point = p1
		p2.Y += Bounds.Height

		graphics.PushTransform(New TranslateTransform(-p1.X, -p1.Y))

		' draw the aspect line
		graphics.DrawLine(New Pen(Stroke, StrokeThickness), p1, p2)

		' find the intersection point with the cause line
		Dim m As Double = (causeNode.P2.Y - causeNode.P1.Y) / (causeNode.P2.X - causeNode.P1.X)
		Dim b As Double = causeNode.P2.Y - m * causeNode.P2.X
		Dim yP As Double = m * p1.X + b

		' draw the label
		If Math.Abs(p1.Y - yP) > Math.Abs(p2.Y - yP) Then
			graphics.DrawText(New FormattedText(c.ToString(), CultureInfo.CurrentCulture, FlowDirection.LeftToRight, New Typeface(New FontFamily("Trebuchet MS"), FontStyles.Normal, FontWeights.Bold, FontStretches.Normal), 12, Brushes.Blue), New Point(p1.X - 3, p1.Y - 15))
		Else
			graphics.DrawText(New FormattedText(c.ToString(), CultureInfo.CurrentCulture, FlowDirection.LeftToRight, New Typeface(New FontFamily("Trebuchet MS"), FontStyles.Normal, FontWeights.Bold, FontStretches.Normal), 12, Brushes.Blue), New Point(p2.X - 3, p2.Y))
		End If

		graphics.Pop()
	End Sub

	Protected Overrides Sub StartCreate(ByVal org As Point)
		MyBase.StartCreate(org)
		start = org
	End Sub

	Protected Overrides Sub UpdateCreate(ByVal current As Point)
		MyBase.UpdateCreate(current)

		Dim dy As Double = start.Y - current.Y
		Bounds = New Rect(New Point(start.X - 0.5, start.Y), New Point(start.X + 0.5, start.Y - dy))
	End Sub

	Protected Overrides Function AllowCreate(ByVal current As Point, ByVal ist As InteractionState) As Boolean
		Return True
	End Function

	Protected Overrides Sub CompleteCreate(ByVal [end] As Point)
		MyBase.CompleteCreate([end])
		AttachTo(causeNode, AttachToNode.TopLeft)
		causeNode.SubordinateGroup.AutoDeleteItems = True
	End Sub


	Private causeNode As CauseNode
	Private start As Point
End Class