﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.Globalization


Public Class CauseNode
	Inherits DiagramNode
	Public Sub New(ByVal backbone As DiagramNode)
		Me.backbone = backbone

		Stroke = Brushes.Red
		StrokeThickness = 3
		Brush = Brushes.Red
		HandlesStyle = HandlesStyle.Invisible
	End Sub

	Public Overrides Function GetRepaintRect(ByVal includeConnected As Boolean) As Rect
		Dim baseRect As Rect = MyBase.GetRepaintRect(includeConnected)
		baseRect.Inflate(2, 2)
		Return baseRect
	End Function

	Public Overrides Sub Draw(ByVal graphics As DrawingContext, ByVal options As MindFusion.Diagramming.Wpf.RenderOptions)
		Dim code As Integer = AscW("a"c) + Math.Max(0, (If(ZIndex = 0, Parent.Items.Count, ZIndex)) - 1)
		Dim c As Char = ChrW(code)
		Dim handle As Point

		Dim p1 As Point = Me.P1
		Dim p2 As Point = Me.P2

		graphics.PushTransform(New TranslateTransform(-Math.Min(p1.X, p2.X), -Math.Min(p1.Y, p2.Y)))

		' draw the label
		If Bounds.Y < backbone.Bounds.Y Then
			graphics.DrawText(New FormattedText(c.ToString(), CultureInfo.CurrentCulture, FlowDirection.LeftToRight, New Typeface(New FontFamily("Trebuchet MS"), FontStyles.Normal, FontWeights.Bold, FontStretches.Normal), 12, Brushes.Blue), New Point(p1.X - 10, p1.Y - 10))
			handle = p2
		Else
			graphics.DrawText(New FormattedText(c.ToString(), CultureInfo.CurrentCulture, FlowDirection.LeftToRight, New Typeface(New FontFamily("Trebuchet MS"), FontStyles.Normal, FontWeights.Bold, FontStretches.Normal), 12, Brushes.Blue), New Point(p2.X - 10, p2.Y - 10))
			handle = p1
		End If

		' draw the cause line
		graphics.DrawLine(New Pen(Stroke, StrokeThickness), p1, p2)

		' draw the movement handle
		graphics.DrawEllipse(Brush, Nothing, New Point(handle.X, handle.Y), 6, 6)

		graphics.Pop()
	End Sub

	Public Overrides Function ContainsPoint(ByVal point As Point, ByVal threshold As Double) As Boolean
		Return Bounds.Contains(point) AndAlso Utilities.DistToLineSegment(point, P1, P2) <= 5 + threshold
	End Function

	Public Overrides Function ContainsPoint(ByVal point As Point) As Boolean
		Return Bounds.Contains(point) AndAlso Utilities.DistToLineSegment(point, P1, P2) <= 5
	End Function

	Protected Overrides Sub StartCreate(ByVal org As Point)
		MyBase.StartCreate(org)
		start = org
		start.Y = backbone.Bounds.Y + backbone.Bounds.Height / 2
	End Sub

	Protected Overrides Sub UpdateCreate(ByVal current As Point)
		MyBase.UpdateCreate(current)

		Dim dy As Double = start.Y - current.Y

		If dy < 0 Then
			' below the backbone
			Bounds = New Rect(New Point(start.X + dy, start.Y), New Point(start.X, start.Y - dy))
		Else
			' above the backbone
			Bounds = New Rect(New Point(start.X - dy, start.Y), New Point(start.X, start.Y - dy))
		End If
	End Sub

	''' <summary>
	''' The upper point.
	''' </summary>
	Public ReadOnly Property P1() As Point
		Get
			Dim p As Point

			If Bounds.Y < backbone.Bounds.Y Then
				p = Bounds.Location
			Else
				p = Bounds.Location
				p.X += Bounds.Width
			End If

			Return p
		End Get
	End Property

	''' <summary>
	''' Thr lower point.
	''' </summary>
	Public ReadOnly Property P2() As Point
		Get
			Dim p As Point

			If Bounds.Y < backbone.Bounds.Y Then
				p = Bounds.Location
				p.X += Bounds.Width
				p.Y += Bounds.Height
			Else
				p = Bounds.Location
				p.Y += Bounds.Height
			End If

			Return p
		End Get
	End Property


	Friend backbone As DiagramNode
	Private start As Point
End Class