﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports MindFusion.Diagramming.Wpf.Behaviors


Public Class FishboneBehavior
	Inherits BehaviorBase
	Public Sub New(ByVal view As Diagram, ByVal backbone As DiagramNode)
		MyBase.New(view)
		Me.backbone = backbone
	End Sub

	Public Overrides Function StartDraw(ByVal point As Point) As InteractionState
		Dim cause As CauseNode = TryCast(Diagram.GetNodeAt(point, 3), CauseNode)
		If cause IsNot Nothing Then
			Dim handle As Point = If((cause.Bounds.Y < backbone.Bounds.Y), cause.P2, cause.P1)
			If Math.Abs(handle.X - point.X) <= 6 AndAlso Math.Abs(handle.Y - point.Y) <= 6 Then
                Return New InteractionState(cause, New NodeAdjustmentHandle(NodeHandleType.Move), MindFusion.Diagramming.Wpf.Action.Modify)
			End If

			If CanDrawAspectAt(point) Then
				Dim node As New AspectNode(cause)
                Return New InteractionState(node, New NodeAdjustmentHandle(NodeHandleType.ResizeBottomRight), MindFusion.Diagramming.Wpf.Action.Create)
			End If
		End If

		If backbone.Bounds.Contains(point) Then
			Dim node As New CauseNode(backbone)
            Return New InteractionState(node, New NodeAdjustmentHandle(NodeHandleType.ResizeBottomRight), MindFusion.Diagramming.Wpf.Action.Create)
		End If

		Return Nothing
	End Function

	Public Overrides Function SetMouseCursor(ByVal point As Point, ByRef startInteraction As Boolean) As CursorHint
		Dim cause As CauseNode = TryCast(Diagram.GetNodeAt(point, 3), CauseNode)
		If cause IsNot Nothing Then
			Dim handle As Point = If((cause.Bounds.Y < backbone.Bounds.Y), cause.P2, cause.P1)
			If Math.Abs(handle.X - point.X) <= 6 AndAlso Math.Abs(handle.Y - point.Y) <= 6 Then
				startInteraction = True
				Return CursorHint.Move
			End If
		End If

		If backbone.Bounds.Contains(point) OrElse CanDrawAspectAt(point) Then
			startInteraction = True
			Return CursorHint.DrawLink
		End If

		startInteraction = False
		Return CursorHint.Pointer
	End Function

	Private Function CanDrawAspectAt(ByVal point As Point) As Boolean
		Dim cause As CauseNode = TryCast(Diagram.GetNodeAt(point), CauseNode)
		If cause IsNot Nothing Then
			If cause.Bounds.Y < backbone.Bounds.Y Then
				' above the backbone
				Dim c As Point = cause.Bounds.Location
				Dim dx As Double = point.X - c.X
				Dim dy As Double = point.Y - c.Y

				If Math.Abs(dx - dy) < 3 Then
					Return True
				End If
			Else
				' below the backbone
				Dim c As Point = cause.Bounds.Location
				c.X = cause.Bounds.Right
				Dim dx As Double = point.X - c.X
				Dim dy As Double = -(point.Y - c.Y)

				If Math.Abs(dx - dy) < 3 Then
					Return True
				End If
			End If
		End If
		Return False
	End Function

	Private backbone As DiagramNode
End Class